# -*- coding: utf-8 -*-
"""
Дополнительные параметры управления системой приоритетного пропуска

(c)2019 ООО "РИПАС-СПБ"
www.ripas.ru
"""
from menu import SubMenu
import RoadCentre
from PythonQt import QtGui, QtCore
from gui import osmView, mainWindow, widgetFromUi, showSelectionOnTree
import webbrowser
from threading import Thread

def showError(msg):
    QtGui.QMessageBox.critical(mainWindow, u"Ошибка", msg)

def showTimeline(routes,archive):
    from timeline import findSynchronizerForTimeline, docTimelineWidget
    RoadCentre.plugins.PriorityPlugin.connect('rpcError(const QString &)', showError)
    try:
        for r in routes:
            timeline_data = RoadCentre.plugins.PriorityPlugin.getTimelineData(r.id)
            doc_window = RoadCentre.plugins.TimelinePlugin.createTimelineWidget(mainWindow, int(r.id), timeline_data, archive)
            tlw = doc_window.findChild('TimelineWidget')
            RoadCentre.plugins.PriorityPlugin.registerTimeline(tlw)
            findSynchronizerForTimeline(tlw)
            tl_list = []
            for oid in tlw.getRoutesTlId():
                light = RoadCentre.project.object(oid)
                if light is not None:
                    tl_list.append(light)
            tlw.setRoutesTl(tl_list)
            docTimelineWidget(doc_window)
            break
    finally:
        RoadCentre.plugins.PriorityPlugin.disconnect('rpcError(const QString &)', showError)

class VehicleMenu(SubMenu):
    def __init__(self):
        SubMenu.__init__(self, (u'Транспортное средство', u'Транспортные средства'))
        self.priority = RoadCentre.plugins.PriorityPlugin
        self.routeItems = []
        self.cursorOnMap = False
        a = self.menuMain.addAction(u'Посмотреть трек за сегодня')
        a.connect('triggered()', self.showTrackForToday)
        a = self.menuMain.addAction(u'Посмотреть трек за ...')
        a.connect('triggered()', self.showTrackFor)
        a = self.menuMain.addAction(u'Статистика ...')
        a.connect('triggered()', self.showStatistics)
        a = self.menuMain.addAction(u'Удалить данные')
        a.connect('triggered()', self.removeBoardsData)

    def selectedVehicles(self):
        for o in RoadCentre.kernel.selection():
            if hasattr(o, "className") and o.className() == "PrioVehicle":
                yield o

    def menu(self, obj):
        m = SubMenu.menu(self, obj)
        self.cursorOnMap = osmView.rect.contains(osmView.mapFromGlobal(QtGui.QCursor.pos()))
        self.routeItems = []
        
        routes_menu = QtGui.QMenu(u'Назначить маршруту ...', m)
        gr = QtGui.QActionGroup(m)
        for route in self.priority.getRoutes():
            route_id = route.modelData(0,QtCore.Qt.DisplayRole)
            m_title = route.modelData(1,QtCore.Qt.DisplayRole)
            if not m_title:
                m_title = str(route_id)
            act = routes_menu.addAction(m_title)
            act.setData(route_id)
            gr.addAction(act)
        gr.connect('triggered(QAction*)', self.assignToRoute)

        show_route_act = None
        hide_route_act = None
        show_act = None
        lock_act = None
        assign_act = None
        unassign_act = None
        for veh in self.selectedVehicles():
            route_item = self.priority.findRoute(veh.routeId)
            if route_item is not None:
                if route_item not in self.routeItems:
                    self.routeItems.append(route_item)
                if route_item.visible:
                    if hide_route_act is None:
                        hide_route_act = m.addAction(u'Скрыть маршрут')
                        hide_route_act.connect('triggered()', self.hideRoute)
                else:
                    if show_route_act is None:
                        show_route_act = m.addAction(u'Показать маршрут')
                        show_route_act.connect('triggered()', self.showRoute)
                if unassign_act is None:
                    unassign_act = m.addAction(u'Снять с маршрута')
                    unassign_act.connect('triggered()', self.unassignFromRoute)
                
            if show_act is None and veh.hasGeometry and not self.cursorOnMap:
                show_act = m.addAction(u'Показать на карте')
                show_act.connect('triggered()', self.showOnMap)
            if lock_act is None:
                lock_act = m.addAction(u'Разблокировать' if veh.locked else u'Заблокировать')
                lock_act.connect('triggered()', self.unlockBoard if veh.locked else self.lockBoard)
            if assign_act is None and not routes_menu.isEmpty():
                assign_act = m.addMenu(routes_menu)
        if len(self.routeItems)>0:
            act = m.addAction(u'Лента времени')
            act.connect('triggered()', lambda: showTimeline(self.routeItems,False))
            act = m.addAction(u'Лента времени (архив)')
            act.connect('triggered()', lambda: showTimeline(self.routeItems,True))
        if self.cursorOnMap:
            act = m.addAction(u"Показать в дереве объектов")
            act.connect('triggered()', showSelectionOnTree)


        act = m.addAction(u"Свойства")
        act.connect('triggered()', self.vehProperties)

        return m

    def showOnMap(self):
        for veh in self.selectedVehicles():
            osmView.zoom = 16
            osmView.centerOn(veh.pos)

    def lockBoard(self):
        for veh in self.selectedVehicles():
            self.priority.setLockBoard(veh.name,True)

    def unlockBoard(self):
        for veh in self.selectedVehicles():
            self.priority.setLockBoard(veh.name,False)

    def showRoute(self):
        for route_item in self.routeItems:
            route_item.visible = True

    def hideRoute(self):
        for route_item in self.routeItems:
            route_item.visible = False

    def unassignFromRoute(self):
        for veh in self.selectedVehicles():
            self.priority.unassignBoard(veh.name)

    def assignToRoute(self, act):
        route_id = act.data()
        if route_id is None:
            return
        for veh in self.selectedVehicles():
            self.priority.assignBoard(veh.name, route_id)

    def showTrackForToday(self):
        cur_date = QtCore.QDate.currentDate().toString(QtCore.Qt.ISODate)
        for veh in self.selectedVehicles():
            url = self.priority.getWebUrl()
            if len(url)==0:
                continue
            url += '/track.htm?log=logs/'+cur_date+'/'+veh.name+'.log'
            thread = Thread(target=webbrowser.open, args=(url,))
            thread.start()
            thread.join(0.3)

    def showStatistics(self):
        veh_names = []
        for veh in self.selectedVehicles():
            veh_names.append(veh.name)
        veh_count = len(veh_names)
        if veh_count == 0:
            return
        dlg = widgetFromUi('priority2/vehstat.ui')
        for item in dlg.findChildren('QWidget'):
            dlg.setProperty(item.objectName, item)
        if self.priority.debugMode:
            dlg.stat_table.insertColumn(8)
            dlg.stat_table.setHorizontalHeaderItem(8, QtGui.QTableWidgetItem("Захвачен службой"))
            dlg.stat_table.insertColumn(9)
            dlg.stat_table.setHorizontalHeaderItem(9, QtGui.QTableWidgetItem("Ожидают захвата"))
        col_count = dlg.stat_table.columnCount
        update_timer = QtCore.QTimer()
        update_timer.setInterval(1000)
        def update_data():
            rpc = self.priority.rpcObject()
            remove_veh_list = []
            for veh_name in veh_names:
                brd_stat = rpc.sendCommand("BoardsServer.getBoardData",{"name":veh_name})
                if brd_stat is None:
                    remove_veh_list.append(veh_name)
                    continue
                brd_stat = brd_stat.get('result',{})
                for p_data in brd_stat.get('protocol_data',[]):
                    i_row = None
                    protocol = p_data.get('protocol')
                    if protocol is None:
                        continue
                    for row in range(dlg.stat_table.rowCount):
                        i_name = dlg.stat_table.item(row,0)
                        if i_name is None:
                            continue
                        i_proto = dlg.stat_table.item(row,1)
                        if i_proto is None:
                            continue
                        if i_name.text() == veh_name and i_proto.text() == protocol:
                            i_row = row
                            break
                    if i_row is None:
                        i_row = dlg.stat_table.rowCount
                        dlg.stat_table.insertRow(i_row)
                        dlg.stat_table.setItem(i_row,0,QtGui.QTableWidgetItem(veh_name))
                        dlg.stat_table.resizeColumnToContents(0)
                        dlg.stat_table.setItem(i_row,1,QtGui.QTableWidgetItem(protocol))
                        for col in range(2,col_count):
                            dlg.stat_table.setItem(i_row,col,QtGui.QTableWidgetItem())
                    dlg.stat_table.item(i_row,2).setText(str(p_data.get('avg_packet_delay_sec','?')))
                    dlg.stat_table.item(i_row,3).setText(str(p_data.get('rcv_data_rate','?')))
                    dlg.stat_table.item(i_row,4).setText(str(p_data.get('rcv_packet_rate','?')))
                    dlg.stat_table.item(i_row,5).setText(str(p_data.get('rcv_buf_size','?')))
                    dlg.stat_table.item(i_row,6).setText(str(p_data.get('black_box','?')))
                    dlg.stat_table.item(i_row,7).setText(p_data.get('last_bb_time','?'))
                    if self.priority.debugMode:
                        dlg.stat_table.item(i_row,8).setText(p_data.get('grabbed_by','?'))
                        dlg.stat_table.item(i_row,9).setText(','.join(p_data.get('wait_for_grab',[])))
            for veh_name in remove_veh_list:
                for row in reversed(range(dlg.stat_table.rowCount)):
                    i_name = dlg.stat_table.item(row,0)
                    if i_name is not None and i_name.text()==veh_name:
                        dlg.stat_table.removeRow(row)
        update_timer.timeout.connect(update_data)
        update_data()
        update_timer.start()
        dlg.stat_table.resizeColumnsToContents()
        dlg.stat_table.horizontalHeader().setMinimumHeight(80)
        getattr(dlg, "exec")()
        update_timer.stop()

    def removeBoardsData(self):
        veh_names = []
        for veh in self.selectedVehicles():
            veh_names.append(veh.name)
        veh_count = len(veh_names)
        if veh_count == 0:
            return
        if veh_count == 1:
            q_text = 'Удалить данные о борте %s?' % veh_names[0]
        else:
            q_text = 'Удалить данные о бортах (%d)?' % veh_count
        ans=QtGui.QMessageBox.question(mainWindow,'Вопрос',q_text,QtGui.QMessageBox.Yes|QtGui.QMessageBox.No)
        if ans==QtGui.QMessageBox.Yes:
            self.priority.removeBoardsData(veh_names)

    def showTrackFor(self):
        dlg = QtGui.QDialog(mainWindow)
        dlg.setLayout(QtGui.QVBoxLayout())
        calendar = QtGui.QCalendarWidget(dlg)
        dlg.layout().addWidget(calendar)
        btn_box = QtGui.QDialogButtonBox(dlg)
        btn_box.setOrientation(QtCore.Qt.Horizontal)
        btn_box.setStandardButtons(QtGui.QDialogButtonBox.Ok | QtGui.QDialogButtonBox.Cancel)
        btn_box.connect("accepted()", dlg.accept)
        btn_box.connect("rejected()", dlg.reject)
        dlg.layout().addWidget(btn_box)
        if getattr(dlg, "exec")() == QtGui.QDialog.Accepted:
            date_str = calendar.selectedDate.toString(QtCore.Qt.ISODate)
            for veh in self.selectedVehicles():
                url = self.priority.getWebUrl()
                if len(url)==0:
                    continue
                url += '/track.htm?log=logs/'+date_str+'/'+veh.name+'.log'
                thread = Thread(target=webbrowser.open, args=(url,))
                thread.start()
                thread.join(0.3)

    def vehProperties(self):
        vehicles = list(self.selectedVehicles())
        veh_count = len(vehicles)
        if veh_count == 0:
            return
        veh = vehicles[0]
        dlg = widgetFromUi('priority2/boardprops.ui')
        if veh_count == 1:
            dlg.windowTitle = u'Свойства борта ' + veh.name
        else:
            dlg.windowTitle = u'Свойства нескольких бортов'
        description = dlg.findChild('QLineEdit', 'description')
        locked = dlg.findChild('QCheckBox', 'locked')
        allow_emergency = dlg.findChild('QGroupBox', 'allow_emergency')
        radius = dlg.findChild('QSpinBox', 'radius')
        sweep_angle = dlg.findChild('QSpinBox', 'sweep_angle')
    
        descr = veh.modelData(1,QtCore.Qt.DisplayRole)
        if descr is not None:
            description.text = descr
        if veh.locked:
            locked.checked = True
        if veh.isEmergency:
            allow_emergency.checked = True
        radius.value = veh.evRadius
        sweep_angle.value = veh.evSweep
        if dlg.exec_() != QtGui.QDialog.Accepted:
            return
        for cur_veh in vehicles:
            changed_props = {}
            if description.text != cur_veh.modelData(1,QtCore.Qt.DisplayRole):
                changed_props["description"] = description.text
            if bool(cur_veh.property("locked")) != locked.checked:
                changed_props["locked"] = locked.checked
            if bool(cur_veh.isEmergency) != allow_emergency.checked:
                changed_props["allow_emergency"] = allow_emergency.checked
            if allow_emergency.checked:
                if radius.value != cur_veh.evRadius:
                    changed_props["radius"] = radius.value
                if sweep_angle.value != cur_veh.evSweep:
                    changed_props["sweep"] = sweep_angle.value
            self.priority.changeBoardProperties(cur_veh.name,changed_props)


class RouteMenu(SubMenu):
    def __init__(self):
        SubMenu.__init__(self, (u'Маршрут', u'Маршруты'))
        self.priority = RoadCentre.plugins.PriorityPlugin
        self.cursorOnMap = False
        if hasattr(RoadCentre.plugins, "TimelinePlugin"):
            a = self.menuMain.addAction(u'Лента времени')
            a.connect('triggered()', lambda: self.showTimeline(False))
            a = self.menuMain.addAction(u'Лента времени (архив)')
            a.connect('triggered()', lambda: self.showTimeline(True))
        self.alwaysVisibleAction = self.menuMain.addAction(u'Всегда видимый')
        self.alwaysVisibleAction.checkable = True
        self.alwaysVisibleAction.connect('triggered(bool)', self.alwaysVisible)
        self.useSignalGroupsEvents = self.menuMain.addAction(u'Использовать события от СГ')
        self.useSignalGroupsEvents.checkable = True
        self.useSignalGroupsEvents.connect('triggered(bool)', self.useSGEvents)

        self.selection = None

    def selectedRoutes(self):
        for o in RoadCentre.kernel.selection():
            if hasattr(o, "className") and o.className() == "PrioRoute":
                yield o

    def menu(self, obj):
        m = SubMenu.menu(self, obj)
        self.cursorOnMap = osmView.rect.contains(osmView.mapFromGlobal(QtGui.QCursor.pos()))
        vis_act = False
        use_sg_act = False
        lock_act = None
        show_act = None
        for r in self.selectedRoutes():
            if not vis_act:
                self.alwaysVisibleAction.checked = r.dontHide
                vis_act = True
            if not use_sg_act:
                self.useSignalGroupsEvents.checked = r.useSgEvents
                use_sg_act = True
            if lock_act is None:
                lock_state = r.locked
                if lock_state is not None:
                    lock_act = m.addAction(u'Разблокировать' if lock_state else u'Заблокирвать')
                    lock_act.connect('triggered()', self.unlockRoute if lock_state else self.lockRoute)
            if show_act is None and getattr(r,'hasGeometry',False) and not self.cursorOnMap:
                show_act = m.addAction(u'Показать на карте')
                show_act.connect('triggered()', self.showOnMap)
        if self.cursorOnMap:
            act = m.addAction(u"Показать в дереве объектов")
            act.connect('triggered()', showSelectionOnTree)
        act = m.addAction(u"Свойства")
        act.connect('triggered()', self.routeProperties)
            
        return m

    def showOnMap(self):
        route_list = list(self.selectedRoutes())
        if len(route_list) == 0:
            r.visible = True
            osmView.fitItem(route_list[0])
            return
        bound_rect = QtCore.QRectF()
        for r in route_list:
            r.visible = True
            bound_rect = bound_rect.united(r.sceneBoundingRect())
        osmView.fitRect(bound_rect)

    def alwaysVisible(self, checked):
        for r in self.selectedRoutes():
            r.dontHide = checked

    def lockRoute(self):
        for r in self.selectedRoutes():
            self.priority.setLockRoute(r.id, True)

    def unlockRoute(self):
        for r in self.selectedRoutes():
            self.priority.setLockRoute(r.id, False)

    def showTimeline(self,archive):
        showTimeline(self.selectedRoutes(),archive)

    def useSGEvents(self, checked):
        for r in self.selectedRoutes():
            r.setUseSgEvents(checked)

    def routeProperties(self):
        routes = list(self.selectedRoutes())
        route_count = len(routes)
        if route_count == 0:
            return
        route = routes[0]
        dlg = widgetFromUi('priority2/routeprops.ui')
        if route_count == 1:
            dlg.windowTitle = u'Свойства маршрута %d' % route.id
        else:
            dlg.windowTitle = u'Свойства нескольких маршрутов'
        description = dlg.findChild('QLineEdit', 'description')
        locked = dlg.findChild('QCheckBox', 'locked')
    
        descr = route.modelData(1,QtCore.Qt.DisplayRole)
        if descr is not None:
            description.text = descr
        if route.locked:
            locked.checked = True
        if dlg.exec_() != QtGui.QDialog.Accepted:
            return
        for cur_route in routes:
            changed_props = {}
            if description.text != cur_route.modelData(1,QtCore.Qt.DisplayRole):
                changed_props["description"] = description.text
            if bool(cur_route.property("locked")) != locked.checked:
                changed_props["locked"] = locked.checked
            self.priority.changeRouteProperties(cur_route.id,changed_props)


class VdtMenu(SubMenu):
    def __init__(self):
        SubMenu.__init__(self, (u'Зона вызова', u'Зоны вызова'))
        self.priority = RoadCentre.plugins.PriorityPlugin
        self.selection = None
        self.cursorOnMap = False

    def selectedDetectors(self):
        for o in RoadCentre.kernel.selection():
            if hasattr(o, "className") and o.className() == "PrioVDetector":
                yield o

    def menu(self, obj):
        m = SubMenu.menu(self, obj)
        self.cursorOnMap = osmView.rect.contains(osmView.mapFromGlobal(QtGui.QCursor.pos()))

        lock_act = None
        show_act = None
        tl_prop_act = None
        activate_act = None
        for vdt in self.selectedDetectors():
            if lock_act is None:
                lock_state = vdt.locked
                if lock_state is not None:
                    lock_act = m.addAction(u'Разблокировать' if lock_state else u'Заблокирвать')
                    lock_act.connect('triggered()', self.unlockVdt if lock_state else self.lockVdt)
            if activate_act is None:
                activate_state = vdt.active
                if activate_state is not None:
                    activate_act = m.addAction(u'Деактивировать' if activate_state else u'Активировать')
                    activate_act.connect('triggered()', self.deactivateVdt if activate_state else self.activateVdt)
            if show_act is None and vdt.hasGeometry and not self.cursorOnMap:
                show_act = m.addAction(u'Показать на карте')
                show_act.connect('triggered()', self.showOnMap)
            if tl_prop_act is None and vdt.tlNumber > 0:
                tl_prop_act = m.addAction(u"Параметры управления СО")
                tl_prop_act.connect('triggered()', self.tlcProperties)
        if self.cursorOnMap:
            act = m.addAction(u"Показать в дереве объектов")
            act.connect('triggered()', showSelectionOnTree)
        act = m.addAction(u"Свойства")
        act.connect('triggered()', self.vdtProperties)
        if self.priority.debugMode:
            act = m.addAction(u"Статистика")
            act.connect('triggered()', self.showStatistics)
        return m

    def lockVdt(self):
        for vdt in self.selectedDetectors():
            self.priority.setLockDetector(vdt.id, True)

    def unlockVdt(self):
        for vdt in self.selectedDetectors():
            self.priority.setLockDetector(vdt.id, False)

    def activateVdt(self):
        for vdt in self.selectedDetectors():
            self.priority.setVdtActive(vdt.id, True)

    def deactivateVdt(self):
        for vdt in self.selectedDetectors():
            self.priority.setVdtActive(vdt.id, False)

    def showOnMap(self):
        vdt_list = list(self.selectedDetectors())
        if len(vdt_list) == 0:
            osmView.fitItem(vdt_list[0])
            return
        bound_rect = QtCore.QRectF()
        for vdt in vdt_list:
            bound_rect = bound_rect.united(vdt.sceneBoundingRect())
        osmView.fitRect(bound_rect)

    def vdtProperties(self):
        vdt_list = list(self.selectedDetectors())
        vdt_count = len(vdt_list)
        if vdt_count == 0:
            return
        vdt = vdt_list[0]
        self.priority.connect('rpcError(const QString &)', showError)
        vdt_props = self.priority.getVdtProperties(vdt.id)
        if vdt_props is None or len(vdt_props) == 0:
            return
        dlg = widgetFromUi('priority2/vdtprops.ui')
        name = dlg.findChild('QLineEdit', 'name')
        max_present_time = dlg.findChild('QSpinBox', 'max_present_time')
        max_unconnected_time = dlg.findChild('QSpinBox', 'max_unconnected_time')
        min_repeat_time = dlg.findChild('QSpinBox', 'min_repeat_time')
        tl = dlg.findChild('QSpinBox', 'tl')
        call_by_button = dlg.findChild('QCheckBox', 'call_by_button')
        prog = dlg.findChild('QSpinBox', 'prog')
        phase = dlg.findChild('QSpinBox', 'phase')
        crossWI = dlg.findChild('QSpinBox', 'crossWI')
        crossWO = dlg.findChild('QSpinBox', 'crossWO')
        no_control = dlg.findChild('QRadioButton', 'no_control')
        cond_table = dlg.findChild('QTableWidget', 'cond_table')

        def checkTlId():
            valid = no_control.isChecked() or tl.value != 0
            pal = QtGui.QSpinBox().palette
            if not valid:
                pal.setColor(QtGui.QPalette.Base, QtCore.Qt.red)
            tl.setPalette(pal)

        tl.connect("valueChanged(QString)", checkTlId)
        no_control.connect("toggled(bool)", checkTlId)

        intattr = lambda v: v if v else 0
        intattr2 = lambda v: v if v else -1
        name.text = vdt_props.get("name", "")
        max_present_time.value = intattr(vdt_props.get("max_present_time", 0))
        max_unconnected_time.value = intattr(vdt_props.get("max_unconnected_time", 0))
        min_repeat_time.value = intattr(vdt_props.get("min_repeat_time", 0))
        cond_values = vdt_props.get("deferred_call_data", {})
        tl.value = intattr(vdt_props.get("tl_number", 0))
        call_by_button.checked = vdt_props.get("priority_by_button", False)
        prog.value = intattr(vdt_props.get("sginal_plan_number", 0))
        phase.value = intattr(vdt_props.get("stage_number", 0))
        crossWI.value = intattr2(vdt_props.get("cross_wi", -1))
        crossWO.value = intattr2(vdt_props.get("cross_wo", -1))
        if call_by_button.checked and len(cond_values) > 0:
            dlg.findChild('QRadioButton', 'by_cond').checked = True
        elif prog.value:
            dlg.findChild('QRadioButton', 'prog_btn').checked = True
        elif phase.value:
            dlg.findChild('QRadioButton', 'phase_btn').checked = True
        row = 0
        for phase_id in cond_values:
            cond_table.setItem(row, 0, QtGui.QTableWidgetItem(str(phase_id)))
            prog_id = cond_values[phase_id]
            try:
                prog_id = int(float(prog_id))
                if prog_id <= 0:
                    prog_id = None
            except ValueError:
                prog_id = None
            if prog_id is not None:
                cond_table.setItem(row, 1, QtGui.QTableWidgetItem(str(prog_id)))
            row += 1
        if dlg.exec_() != QtGui.QDialog.Accepted:
            return
        changed_props = {}
        if max_present_time.value != vdt_props.get("max_present_time"):
            changed_props["max_present_time"] = max_present_time.value
        if max_unconnected_time.value != vdt_props.get("max_unconnected_time"):
            changed_props["max_unconnected_time"] = max_unconnected_time.value
        if min_repeat_time.value != vdt_props.get("min_repeat_time"):
            changed_props["min_repeat_time"] = min_repeat_time.value
        if tl.value != vdt_props.get("tl_number"):
            changed_props["tl_number"] = tl.value
        if call_by_button.checked != vdt_props.get("priority_by_button"):
            changed_props["priority_by_button"] = call_by_button.checked
        if crossWI.value != vdt_props.get("cross_wi"):
            changed_props["cross_wi"] = crossWI.value if crossWI.value >= 0 else None
        if crossWO.value != vdt_props.get("cross_wo"):
            changed_props["cross_wo"] = crossWO.value if crossWO.value >= 0 else None
        new_cond_values = {}
        if cond_table.enabled:
            for row in range(cond_table.rowCount):
                item = cond_table.item(row, 0)
                if item is None:
                    continue
                ph_id = item.text()
                if not ph_id.isdigit():
                    continue
                item = cond_table.item(row, 1)
                pr_id = None
                if item is not None:
                    pr_id = item.text()
                    try:
                        pr_id = int(float(pr_id))
                        if pr_id <= 0:
                            pr_id = None
                    except ValueError:
                        pr_id = None
                new_cond_values[int(ph_id)] = pr_id

        if new_cond_values != cond_values:
            changed_props["deferred_call_data"] = new_cond_values
        prog_val = prog.value if prog.enabled else -1
        if prog_val != vdt_props.get("sginal_plan_number"):
            changed_props["sginal_plan_number"] = prog_val
        phase_val = phase.value if phase.enabled else -1
        try:
            if phase_val != vdt_props.get("stage_number"):
                changed_props["stage_number"] = phase_val
            if prog_val == 0:
                showError(u"Необходимо указать номер программы")
                return
            if phase_val == 0:
                showError(u"Необходимо указать номер фазы")
                return
            if name.text != vdt_props.get("name"):
                changed_props["name"] = name.text
            if not len(changed_props):
                return
            self.priority.changeVdtProperties(vdt.id,changed_props)
        finally:
            self.priority.disconnect('rpcError(const QString &)', showError)

    def tlcProperties(self):
        tl_number = None
        dlg = None
        self.priority.connect('rpcError(const QString &)', showError)
        for vdt in self.selectedDetectors():
            if vdt.tlNumber > 0:
                tlc_props = self.priority.getTlcProperties(vdt.tlNumber)
                if tlc_props is None or len(tlc_props) == 0:
                    return
                if tl_number is None:
                    tl_number = vdt.tlNumber
                else:
                    tl_number = None
                    break
                dlg = widgetFromUi('priority2/tlcprops.ui')
                max_duty_time = dlg.findChild('QSpinBox', 'max_duty_time')
                min_relax_time = dlg.findChild('QSpinBox', 'min_relax_time')
                cross_group = dlg.findChild('QGroupBox', 'crossGroup')
                cross_host = dlg.findChild('QLineEdit', 'cross_host')
                cross_port = dlg.findChild('QSpinBox', 'cross_port')
                intattr = lambda v: v if v else 0
                max_duty_time.value = intattr(tlc_props.get('max_duty_time'))
                min_relax_time.value = intattr(tlc_props.get('min_relax_time'))
                cross_host.text = tlc_props.get('cross_host','')
                cross_port.value = intattr(tlc_props.get('cross_port'))
                break
        try:
            if dlg is None:
                return
            if tl_number is None:
                dlg.windowTitle = u"Свойства нескольких СО"
            else:
                if len(cross_host.text)>0:
                    cross_group.checked = True
                dlg.windowTitle = u"Свойства СО %d" % tl_number

            if dlg.exec_() != QtGui.QDialog.Accepted:
                return

            changed_props = {}
            if max_duty_time.value > 0:
                changed_props["max_duty_time"] = max_duty_time.value
            if min_relax_time.value > 0:
                changed_props["min_relax_time"] = min_relax_time.value
            if tl_number is not None:
                if not cross_group.checked or tlc_props.get('cross_host') != cross_host.text:
                    changed_props["cross_host"] = cross_host.text if cross_group.checked else None
                if not cross_group.checked or intattr(tlc_props.get('cross_port')) != cross_port.value:
                    changed_props["cross_port"] = cross_port.value if cross_group.checked else None

            for vdt in self.selectedDetectors():
                if vdt.tlNumber > 0:
                    if len(changed_props) > 0:
                        self.priority.changeTlcProperties(vdt.tlNumber,changed_props)
        finally:
            self.priority.disconnect('rpcError(const QString &)', showError)

    def showStatistics(self):
        vdt_ids = []
        for vdt in self.selectedDetectors():
            vdt_ids.append(vdt.id)
        vdt_count = len(vdt_ids)
        if vdt_count == 0:
            return
        dlg = widgetFromUi('priority2/vdtstat.ui')
        for item in dlg.findChildren('QWidget'):
            dlg.setProperty(item.objectName, item)
        col_count = dlg.stat_table.columnCount
        update_timer = QtCore.QTimer()
        update_timer.setInterval(1000)
        def update_data():
            rpc = self.priority.rpcObject()
            remove_vdt_list = []
            for vdt_id in vdt_ids:
                vdt_stat = rpc.sendCommand("VDTService.getVdtProperties",{"id":vdt_id})
                if vdt_stat is None:
                    remove_vdt_list.append(vdt_id)
                    continue
                vdt_stat = vdt_stat.get('result',{})
                i_row = None
                for row in range(dlg.stat_table.rowCount):
                    i_id = dlg.stat_table.item(row,0)
                    if i_id is None:
                        continue
                    if i_id.text() == str(vdt_id):
                        i_row = row
                        break
                if i_row is None:
                    i_row = dlg.stat_table.rowCount
                    dlg.stat_table.insertRow(i_row)
                    dlg.stat_table.setItem(i_row,0,QtGui.QTableWidgetItem(str(vdt_id)))
                    dlg.stat_table.resizeColumnToContents(0)
                    dlg.stat_table.setItem(i_row,1,QtGui.QTableWidgetItem(vdt_stat.get('name','?')))
                    for col in range(2,col_count):
                        dlg.stat_table.setItem(i_row,col,QtGui.QTableWidgetItem())
                dlg.stat_table.item(i_row,2).setText(vdt_stat.get('state','?'))
                dlg.stat_table.item(i_row,3).setText(str(vdt_stat.get('on_time','?')))
                dlg.stat_table.item(i_row,4).setText(str(vdt_stat.get('tl_number','?')))
                dlg.stat_table.item(i_row,5).setText(vdt_stat.get('board_name','?'))
                dlg.stat_table.item(i_row,6).setText(str(vdt_stat.get('board_time','?')))
                dlg.stat_table.item(i_row,7).setText(vdt_stat.get('grabbed_by','?'))
            for vdt_id in remove_vdt_list:
                for row in reversed(range(dlg.stat_table.rowCount)):
                    i_id = dlg.stat_table.item(row,0)
                    if i_id is not None and i_id.text()==str(vdt_id):
                        dlg.stat_table.removeRow(row)
        update_timer.timeout.connect(update_data)
        update_data()
        update_timer.start()
        dlg.stat_table.resizeColumnsToContents()
        dlg.stat_table.horizontalHeader().setMinimumHeight(80)
        getattr(dlg, "exec")()
        update_timer.stop()
